#!/bin/bash
# install full text search
# https://www.c-rieger.de/volltextsuche-mit-nextcloud-elasticsearch-und-tessaract/
#
# elastichsearch prüfen:
# sudo -u www-data php /var/www/html/nextcloud/occ fulltextsearch:test

SOURCE=`dirname $0`
# install elasticsearch
apt install apt-transport-https ca-certificates openjdk-8-jre -y >> $SOURCE/install.out 2> $SOURCE/install.err
curl -fsSL https://artifacts.elastic.co/GPG-KEY-elasticsearch |sudo gpg --dearmor -o /usr/share/keyrings/elastic.gpg >> $SOURCE/install.out 2> $SOURCE/install.err
echo "deb [signed-by=/usr/share/keyrings/elastic.gpg] https://artifacts.elastic.co/packages/7.x/apt stable main" | sudo tee -a /etc/apt/sources.list.d/elastic-7.x.list >> $SOURCE/install.out 2> $SOURCE/install.err
apt update >> $SOURCE/install.out 2> $SOURCE/install.err
apt install elasticsearch -y >> $SOURCE/install.out 2> $SOURCE/install.err
systemctl daemon-reload >> $SOURCE/install.out 2> $SOURCE/install.err
systemctl enable elasticsearch >> $SOURCE/install.out 2> $SOURCE/install.err
systemctl start elasticsearch >> $SOURCE/install.out 2> $SOURCE/install.err

# plugin: search in pdf files
/usr/share/elasticsearch/bin/elasticsearch-plugin install ingest-attachment -b >> $SOURCE/install.out 2> $SOURCE/install.err

# config
echo "network.host: 127.0.0.1" >> /etc/elasticsearch/elasticsearch.yml >> $SOURCE/install.out 2> $SOURCE/install.err
echo "ES_JAVA_OPTS=\"-Xms2g -Xmx2g\"" >> /etc/default/elasticsearch >> $SOURCE/install.out 2> $SOURCE/install.err

cp nextcloud-elasticsearch-worker.service /etc/systemd/system/ >> $SOURCE/install.out 2> $SOURCE/install.err
systemctl daemon-reload >> $SOURCE/install.out 2> $SOURCE/install.err
systemctl restart elasticsearch.service nextcloud-elasticsearch-worker.service >> $SOURCE/install.out 2> $SOURCE/install.err

# plugin: search pics
apt install tesseract-ocr tesseract-ocr-deu tesseract-ocr-eng -y >> $SOURCE/install.out 2> $SOURCE/install.err
service elasticsearch restart >> $SOURCE/install.out 2> $SOURCE/install.err

# install apps
# Full text search
sudo -u www-data php /var/www/html/nextcloud/occ app:install fulltextsearch >> $SOURCE/install.out 2> $SOURCE/install.err
sudo -u www-data php /var/www/html/nextcloud/occ app:enable fulltextsearch >> $SOURCE/install.out 2> $SOURCE/install.err
# Full text search - Elasticsearch Platform
sudo -u www-data php /var/www/html/nextcloud/occ app:install fulltextsearch_elasticsearch >> $SOURCE/install.out 2> $SOURCE/install.err
sudo -u www-data php /var/www/html/nextcloud/occ app:enable fulltextsearch_elasticsearch >> $SOURCE/install.out 2> $SOURCE/install.err
# Full text search - Files
sudo -u www-data php /var/www/html/nextcloud/occ app:install files_fulltextsearch >> $SOURCE/install.out 2> $SOURCE/install.err
sudo -u www-data php /var/www/html/nextcloud/occ app:enable files_fulltextsearch >> $SOURCE/install.out 2> $SOURCE/install.err
# Full text search - Files - Tesseract OCR
sudo -u www-data php /var/www/html/nextcloud/occ app:install files_fulltextsearch_tesseract >> $SOURCE/install.out 2> $SOURCE/install.err
sudo -u www-data php /var/www/html/nextcloud/occ app:enable files_fulltextsearch_tesseract >> $SOURCE/install.out 2> $SOURCE/install.err

# config app elasticsearch
cp elasticsearch.json /tmp >> $SOURCE/install.out 2> $SOURCE/install.err
chmod a=rw /tmp/elasticsearch.json >> $SOURCE/install.out 2> $SOURCE/install.err
sudo -u www-data php /var/www/html/nextcloud/occ config:import /tmp/elasticsearch.json >> $SOURCE/install.out 2> $SOURCE/install.err
# create index
sudo -u www-data php /var/www/html/nextcloud/occ fulltextsearch:index
