#!/bin/bash
cd nextcloud
SOURCE=.
# get os
OS="U"
if (cat /etc/issue | grep -i "RASPBIAN")
then
  OS="R"
fi
echo "System:" $OS

# install dialog
apt-get update > $SOURCE/install.out 2> $SOURCE/install.err
apt-get upgrade -y >> $SOURCE/install.out 2>> $SOURCE/install.err
apt-get -fym install dialog >> $SOURCE/install.out 2>> $SOURCE/install.err

# ask for domain and email address
DOMAIN=""
MAIL=""
INPUT="/tmp/input.txt"
dialog --title  "Installation Nextcloud" --menu "" 10 40 2 1 "External use" 2 "Internal use only"  2>$INPUT
response=$?
MENU=$(cat $INPUT)
if [ "$response" = "1" ]
then
  exit
fi
if [ "$response" = "255" ]
then
  exit
fi

if [ "$MENU" = "1" ]
then
  dialog --title "Installation Nextcloud" --inputbox "Domain name" 10 40 2>$INPUT
  response=$?
  DOMAIN=$(cat $INPUT)
  if [ "$response" = "1" ]
  then
    exit
  fi
  if [ "$response" = "255" ]
  then
    exit
  fi
  if [ "$DOMAIN" = "" ]
  then
    exit
  fi

  dialog --title "Installation Nextcloud" --inputbox "Email address" 10 40 2>$INPUT
  MAIL=$(cat $INPUT)
  response=$?
  if [ "$response" = "1" ]
  then
    exit
  fi
  if [ "$response" = "255" ]
  then
    exit
  fi
  if [ "$DOMAIN" = "" ]
  then
    exit
  fi
fi

# Database password
dialog --title "Installation Nextcloud" --inputbox "Database password" 10 40 2>$INPUT
DBPASSWORD=$(cat $INPUT)
response=$?
if [ "$response" = "1" ]
then
  exit
fi
if [ "$response" = "255" ]
then
  exit
fi
if [ "$DBPASSWORD" = "" ]
then
  exit
fi

# admin password nextcloud
dialog --title "Installation Nextcloud" --inputbox "Password Nextcloud admin" 10 40 2>$INPUT
ADMINPASSWORD=$(cat $INPUT)
response=$?
if [ "$response" = "1" ]
then
  exit
fi
if [ "$response" = "255" ]
then
  exit
fi
if [ "$ADMINPASSWORD" = "" ]
then
  exit
fi

echo "* start installation"
echo "1. install packages"
apt-get update > $SOURCE/install.out 2>> $SOURCE/install.err
apt-get upgrade -y >> $SOURCE/install.out 2>> $SOURCE/install.err
# install Webserver
apt-get install apache2 -y >> $SOURCE/install.out 2>> $SOURCE/install.err
apt-get install zip imagemagick -y >> $SOURCE/install.out 2>> $SOURCE/install.err

if [ "$OS" = "U" ]
then
    apt-get install php zip libapache2-mod-php php-gd php-json php-mysql php-curl php-mbstring php-intl php-imagick imagemagick php-xml php-zip php-mysql php-bcmath php-gmp -y >> $SOURCE/install.out 2>> $SOURCE/install.err
    apt-get install -y mysql-server mysql-client python3-mysql.connector >> $SOURCE/install.out 2>> $SOURCE/install.err
    apt-get install -y pip >> $SOURCE/install.out 2>> $SOURCE/install.err
else
# PHP 8
    wget -qO /etc/apt/trusted.gpg.d/php.gpg https://packages.sury.org/php/apt.gpg >> $SOURCE/install.out 2>> $SOURCE/install.err
    echo "deb https://packages.sury.org/php/ $(lsb_release -sc) main" | sudo tee /etc/apt/sources.list.d/php.list >> $SOURCE/install.out 2>> $SOURCE/install.err
    apt-get update >> $SOURCE/install.out 2>> $SOURCE/install.err
    apt-get install php libapache2-mod-php php-gd php-json php-mysql php-curl php-mbstring php-intl php-imagick php-xml php-zip php-mysql php-bcmath php-gmp -y >> $SOURCE/install.out 2>> $SOURCE/install.err
    a2dismod php7.4
    a2enmod php8.2
    systemctl restart apache2
    apt-get autoremove -y
    apt-get install -y python3-pip >> $SOURCE/install.out 2>> $SOURCE/install.err
    apt-get install -y mariadb-server mariadb-client libmariadb-dev >> $SOURCE/install.out 2>> $SOURCE/install.err
    pip3 install packaging >> $SOURCE/install.out 2>> $SOURCE/install.err
    pip3 install mariadb >> $SOURCE/install.out 2>> $SOURCE/install.err
fi

apt-get install sshguard -y >> $SOURCE/install.out 2>> $SOURCE/install.err
apt-get install fail2ban -y >> $SOURCE/install.out 2>> $SOURCE/install.err
# letsencrypt
apt-get install certbot python3-certbot-apache -y >> $SOURCE/install.out 2>> $SOURCE/install.err
# coturn
apt-get install coturn -y >> $SOURCE/install.out 2>> $SOURCE/install.err
# Firewall and automatic updates
apt-get install ufw unattended-upgrades apt-config-auto-update -y >> $SOURCE/install.out 2>> $SOURCE/install.err
# sshpass
apt-get install sshpass
# postfix
#apt-get install postfix -y

echo "2. config database"
sed -i "s/DBPASSWORD/$DBPASSWORD/g" config.mysql >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "s/DBPASSWORD/$DBPASSWORD/g" nextcloud.mysql >> $SOURCE/install.out 2>> $SOURCE/install.err

DATABASE=mysql
if [ "$OS" = "R" ]
then
    DATABASE=mariadb
fi
$DATABASE < config.mysql >> $SOURCE/install.out 2>> $SOURCE/install.err
$DATABASE -uroot -p"$DBPASSWORD" < nextcloud.mysql >> $SOURCE/install.out 2>> $SOURCE/install.err

echo "3. install nextcloud"
wget https://download.nextcloud.com/server/releases/latest.zip >> $SOURCE/install.out 2>> $SOURCE/install.err
unzip latest.zip >> $SOURCE/install.out 2>> $SOURCE/install.err
mv nextcloud /var/www/html >> $SOURCE/install.out 2>> $SOURCE/install.err
chown -R www-data:www-data /var/www/html/nextcloud >> $SOURCE/install.out 2>> $SOURCE/install.err

echo "4. create certificate"
if [ "$DOMAIN" != "" ]
then
  systemctl stop apache2 >> $SOURCE/install.out 2>> $SOURCE/install.err
  certbot -m $MAIL -d $DOMAIN --noninteractive --standalone --agree-tos certonly >> $SOURCE/install.out 2>> $SOURCE/install.err
  systemctl start apache2 >> $SOURCE/install.out 2>> $SOURCE/install.err
fi

echo "5. config apache"
cp /etc/apache2/apache2.conf apache.conf.backup >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "s/Options FollowSymLinks/Options -Indexes -FollowSymLinks -Includes -ExecCGI/g" /etc/apache2/apache2.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "s/Options Indexes FollowSymLinks/Options -Indexes -FollowSymLinks -Includes -ExecCGI/g" /etc/apache2/apache2.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "/^<Directory \/>/a    Order deny,allow" /etc/apache2/apache2.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "/^<Directory \/>/a    Require all denied" /etc/apache2/apache2.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "/^<Directory \/var\/www\/>/a    Order deny,allow" /etc/apache2/apache2.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "/^<Directory \/var\/www\/>/a    Require all denied" /etc/apache2/apache2.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "/^<Directory \/var\/www\/>/a    FileEtag None" /etc/apache2/apache2.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "/^<Directory \/var\/www\/>/a    Header always append X-Frame-Options SAMEORIGIN" /etc/apache2/apache2.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i '/^<Directory \/var\/www\/>/a    Header set X-XSS-Protection "1; mode=block' /etc/apache2/apache2.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "s|^<Directory \/var\/www\/>|<Directory /var/www/html/nextcloud/>|g" /etc/apache2/apache2.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
cp /etc/apache2/conf-enabled/security.conf security.conf.backup >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "s/^ServerSignature On/ServerSignature Off/g" /etc/apache2/conf-enabled/security.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "s/^ServerTokens OS/ServerTokens Prod/g" /etc/apache2/conf-enabled/security.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
cp nextcloud.conf /etc/apache2/sites-available/ >> $SOURCE/install.out 2>> $SOURCE/install.err
if [ "$DOMAIN" != "" ]
then
  sed -i "s/DOMAIN/$DOMAIN/g" /etc/apache2/sites-available/nextcloud.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
  sed -i "s/IPADDRESS/$DOMAIN/g" /etc/apache2/sites-available/nextcloud.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
  echo "ServerName" $DOMAIN >> /etc/apache2/apache2.conf
else
  sed -i "/DOMAIN/d" /etc/apache2/sites-available/nextcloud.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
  ip=$(hostname --all-ip | cut -f1 -d" ")
  sed -i "s/IPADDRESS/$ip/g" /etc/apache2/sites-available/nextcloud.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
  sed -i "s/#SSLCertificateFile/SSLCertificateFile/g" /etc/apache2/sites-available/nextcloud.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
  echo "ServerName" 127.0.0.1 >> /etc/apache2/apache2.conf
fi

cp nextcloud.pem /etc/ssl/certs >> $SOURCE/install.out 2>> $SOURCE/install.err
a2enmod ssl >> $SOURCE/install.out 2>> $SOURCE/install.err
a2ensite nextcloud >> $SOURCE/install.out 2>> $SOURCE/install.err
a2dissite 000-default.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
a2enmod rewrite headers env dir mime >> $SOURCE/install.out 2>> $SOURCE/install.err
a2enmod actions >> $SOURCE/install.out 2>> $SOURCE/install.err
a2dismod status >> $SOURCE/install.out 2>> $SOURCE/install.err
systemctl restart apache2

if [ "$OS" = "R" ]
then
  sed -i '/^memory_limit =/s/=.*/= 512M/' /etc/php/8.2/apache2/php.ini >> $SOURCE/install.out 2>> $SOURCE/install.err
else
  sed -i '/^memory_limit =/s/=.*/= 512M/' /etc/php/8.1/apache2/php.ini >> $SOURCE/install.out 2>> $SOURCE/install.err
fi

echo "6. config backup"
adduser --disabled-password --gecos "" syncnextcloud >> $SOURCE/install.out 2>> $SOURCE/install.err
echo syncnextcloud:sync2Next | chpasswd >> $SOURCE/install.out 2>> $SOURCE/install.err
mkdir -p /work/system >> $SOURCE/install.out 2>> $SOURCE/install.err
mkdir -p /work/backup >> $SOURCE/install.out 2>> $SOURCE/install.err
mkdir -p /work/sync >> $SOURCE/install.out 2>> $SOURCE/install.err
chown syncnextcloud:syncnextcloud /work/sync
mkdir -p /work/restore >> $SOURCE/install.out 2>> $SOURCE/install.err
cp backup.sh /work/system >> $SOURCE/install.out 2>> $SOURCE/install.err
cp sync.sh /work/system >> $SOURCE/install.out 2>> $SOURCE/install.err
cp restore.sh /work/system >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "s/DBPASSWORD/$DBPASSWORD/g" /work/system/sync.sh >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "s/DBPASSWORD/$DBPASSWORD/g" /work/system/restore.sh >> $SOURCE/install.out 2>> $SOURCE/install.err
chown root:root /work/system/sync.sh
chmod u=rwx,g=x,o=x /work/system/sync.sh
chown root:root /work/system/restore.sh
chmod u=rwx,g=x,o=x /work/system/restore.sh

echo "7. config renew script for letsencrypt"
if [ "$DOMAIN" != "" ]
then
  sed -i "s/DOMAIN/$DOMAIN/g" ./newcert.sh >> $SOURCE/install.out 2>> $SOURCE/install.err
  cp newcert.sh /work/system >> $SOURCE/install.out 2>> $SOURCE/install.err
else
  sed -i "/newcert.sh/d" crontab >> $SOURCE/install.out 2>> $SOURCE/install.err
fi

cp crontab /work/system >> $SOURCE/install.out 2>> $SOURCE/install.err
cp crontab_sync /work/system >> $SOURCE/install.out 2>> $SOURCE/install.err
cp crontab_restore /work/system >> $SOURCE/install.out 2>> $SOURCE/install.err
cp sshkey.sh /work/system >> $SOURCE/install.out 2>> $SOURCE/install.err
crontab /work/system/crontab >> $SOURCE/install.out 2>> $SOURCE/install.err

#echo "config turnserver"
#sudo sed -i '/TURNSERVER_ENABLED/c\TURNSERVER_ENABLED=1' /etc/default/coturn >> $SOURCE/install.out 2>> $SOURCE/install.err
#setcap cap_net_bind_service=+ep /usr/bin/turnserver >> $SOURCE/install.out 2>> $SOURCE/install.err
#mkdir /etc/coturn >> $SOURCE/install.out 2>> $SOURCE/install.err
#chmod u=rwx,g=rx,o=-rwx /etc/coturn >> $SOURCE/install.out 2>> $SOURCE/install.err
#cp /etc/letsencrypt/live/$DOMAIN/fullchain.pem /etc/coturn >> $SOURCE/install.out 2>> $SOURCE/install.err
#cp /etc/letsencrypt/live/$DOMAIN/privkey.pem /etc/coturn >> $SOURCE/install.out 2>> $SOURCE/install.err
#chown -R turnserver:turnserver /etc/coturn >> $SOURCE/install.out 2>> $SOURCE/install.err
#cp /etc/turnserver.conf turnserver.conf.bak >> $SOURCE/install.out 2>> $SOURCE/install.err
#sed -s 's/DOMAIN/$DOMAIN/g' ./turnserver.conf >> $SOURCE/install.out 2>> $SOURCE/install.err
#cat /etc/turnserver.conf turnserver.conf > turnserver.new >> $SOURCE/install.out 2>> $SOURCE/install.err
#mv turnserver.new /etc/turnserver.conf >> $SOURCE/install.out 2>> $SOURCE/install.err

echo "8. init database"
sudo -u www-data php /var/www/html/nextcloud/occ maintenance:install --database "mysql" --database-name "nextcloud" --database-user "nextcloud" --database-pass "$DBPASSWORD" --admin-user "admin" --admin-pass "$ADMINPASSWORD" >> $SOURCE/install.out 2>> $SOURCE/install.err

echo "9. config php"
cp /var/www/html/nextcloud/config/config.php config.php.orig >> $SOURCE/install.out 2>> $SOURCE/install.err
sed -i "/);/i\\  'skeletondirectory' => '\/work\/data\/skeleton'," /var/www/html/nextcloud/config/config.php >> $SOURCE/install.out 2>> $SOURCE/install.err
# get first ip address from system
ip=$(hostname --all-ip | cut -f1 -d" ")
if [ "$DOMAIN" != "" ]
then
  sed -i "/0 => 'localhost',/a \\\t  2 => '$ip'," /var/www/html/nextcloud/config/config.php >> $SOURCE/install.out 2>> $SOURCE/install.err
  sed -i "/0 => 'localhost',/a \\\t  1 => '$DOMAIN'," /var/www/html/nextcloud/config/config.php >> $SOURCE/install.out 2>> $SOURCE/install.err
else
  sed -i "/0 => 'localhost',/a \\\t  1 => '$ip'," /var/www/html/nextcloud/config/config.php >> $SOURCE/install.out 2>> $SOURCE/install.err
fi

echo "10. config working directory"
cp -a /var/www/html/nextcloud/data /work >> $SOURCE/install.out 2>> $SOURCE/install.err
chown -R www-data:www-data /work/data >> $SOURCE/install.out 2>> $SOURCE/install.err
rm -r /var/www/html/nextcloud/data >> $SOURCE/install.out 2>> $SOURCE/install.err
ln -s /work/data /var/www/html/nextcloud/data >> $SOURCE/install.out 2>> $SOURCE/install.err
chown -R www-data:www-data /var/www/html/nextcloud >> $SOURCE/install.out 2>> $SOURCE/install.err
# new skeleton folder: there is a problem writing office file from original template
cp -a /var/www/html/nextcloud/core/skeleton /work/data >> $SOURCE/install.out 2>> $SOURCE/install.err
rm /work/data/skeleton/Templates/* >> $SOURCE/install.out 2>> $SOURCE/install.err
cp leer.* /work/data/skeleton/Templates >> $SOURCE/install.out 2>> $SOURCE/install.err
chown -R www-data:www-data /work/data/skeleton >> $SOURCE/install.out 2>> $SOURCE/install.err

echo "11. install apps"
sh ./installapps.sh
echo "12. install elasticsearch"
sh ./elasticsearch.sh

echo "13. config firewall"
ufw allow ssh
ufw allow http
ufw allow https
ufw enable

if [ -s $SOURCE/install.err ]
then
  echo "*********************************************"
  echo "error installation (see $SOURCE/install.err):"
  cat $SOURCE/install.err
else
  echo
  echo "* Reboot system now"
fi
